package com.livelike.flickersearchlibrary.service.logic

import android.util.Log
import com.livelike.flickersearchlibrary.api.FlickrApi
import com.livelike.flickersearchlibrary.api.model.Photo
import com.livelike.flickersearchlibrary.service.FlickrService

/**
 * Implementation of [FlickrService].
 */
internal class FlickrServiceLogic(private val api: FlickrApi) : FlickrService {

    private val cachedPhotos = mutableListOf<Photo>()

    private val LOG_TAG = "FlickrService"

    override suspend fun search(searchQuery: String?) =
        searchQuery?.let {
            updateCachedPhotos(searchQuery)
            cachedPhotos
        } ?: emptyList()

    private suspend fun updateCachedPhotos(searchQuery: String) =
        runCatching {
            val response = api.search(searchQuery)
            if (response.isSuccessful) {
                response.body()?.let {
                    cachedPhotos.clear()
                    cachedPhotos.addAll(it.photosPage.photos)
                }
            } else {
                Log.d(
                    LOG_TAG,
                    "Flickr API responded with error: ${response.errorBody().toString()}"
                )
            }
        }.onFailure { exception: Throwable ->
            Log.d(
                LOG_TAG,
                "Exception occurred when executing API call: ${exception.message}"
            )
        }
}